/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import fs from 'node:fs/promises';
import { Flags, SfCommand, loglevel } from '@salesforce/sf-plugins-core';
import { AuthInfo, Messages } from '@salesforce/core';
import { parseJson } from '@salesforce/kit';
import common from '../../../common.js';
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-auth', 'sfdxurl.store');
const commonMessages = Messages.loadMessages('@salesforce/plugin-auth', 'messages');
const AUTH_URL_FORMAT = 'force://<clientId>:<clientSecret>:<refreshToken>@<instanceUrl>';
export default class LoginSfdxUrl extends SfCommand {
    static summary = messages.getMessage('summary');
    static description = messages.getMessage('description', [AUTH_URL_FORMAT]);
    static examples = messages.getMessages('examples');
    static aliases = ['force:auth:sfdxurl:store', 'auth:sfdxurl:store'];
    static deprecateAliases = true;
    static flags = {
        'sfdx-url-file': Flags.file({
            char: 'f',
            summary: messages.getMessage('flags.sfdx-url-file.summary'),
            required: false,
            deprecateAliases: true,
            aliases: ['sfdxurlfile'],
            exactlyOne: ['sfdx-url-file', 'sfdx-url-stdin'],
        }),
        'sfdx-url-stdin': Flags.file({
            char: 'u',
            summary: messages.getMessage('flags.sfdx-url-stdin.summary'),
            required: false,
            deprecateAliases: true,
            aliases: ['sfdxurlstdin'],
            allowStdin: 'only',
            exactlyOne: ['sfdx-url-file', 'sfdx-url-stdin'],
        }),
        'set-default-dev-hub': Flags.boolean({
            char: 'd',
            summary: commonMessages.getMessage('flags.set-default-dev-hub.summary'),
            deprecateAliases: true,
            aliases: ['setdefaultdevhub', 'setdefaultdevhubusername'],
        }),
        'set-default': Flags.boolean({
            char: 's',
            summary: commonMessages.getMessage('flags.set-default.summary'),
            deprecateAliases: true,
            aliases: ['setdefaultusername'],
        }),
        alias: Flags.string({
            char: 'a',
            summary: commonMessages.getMessage('flags.alias.summary'),
            deprecateAliases: true,
            aliases: ['setalias'],
        }),
        'no-prompt': Flags.boolean({
            char: 'p',
            summary: commonMessages.getMessage('flags.no-prompt.summary'),
            required: false,
            hidden: true,
            deprecateAliases: true,
            aliases: ['noprompt'],
        }),
        loglevel,
    };
    async run() {
        const { flags } = await this.parse(LoginSfdxUrl);
        if (await common.shouldExitCommand(flags['no-prompt']))
            return {};
        const authFile = flags['sfdx-url-file'];
        const authStdin = flags['sfdx-url-stdin'];
        let sfdxAuthUrl;
        if (authFile) {
            sfdxAuthUrl = authFile.endsWith('.json') ? await getUrlFromJson(authFile) : await fs.readFile(authFile, 'utf8');
            if (!sfdxAuthUrl) {
                throw new Error(`Error getting the SFDX authorization URL from file ${authFile}. Ensure it meets the description shown in the documentation (--help) for this command.`);
            }
        }
        else if (authStdin) {
            sfdxAuthUrl = authStdin;
        }
        else {
            throw new Error('SFDX authorization URL not found.');
        }
        const oauth2Options = AuthInfo.parseSfdxAuthUrl(sfdxAuthUrl);
        const authInfo = await AuthInfo.create({ oauth2Options });
        await authInfo.save();
        await authInfo.handleAliasAndDefaultSettings({
            alias: flags.alias,
            setDefault: flags['set-default'],
            setDefaultDevHub: flags['set-default-dev-hub'],
        });
        // ensure the clientSecret field... even if it is empty
        const result = { clientSecret: '', ...authInfo.getFields(true) };
        await AuthInfo.identifyPossibleScratchOrgs(result, authInfo);
        const successMsg = commonMessages.getMessage('authorizeCommandSuccess', [result.username, result.orgId]);
        this.logSuccess(successMsg);
        return result;
    }
}
const getUrlFromJson = async (authFile) => {
    const jsonContents = await fs.readFile(authFile, 'utf8');
    const authFileJson = parseJson(jsonContents);
    return authFileJson.result?.sfdxAuthUrl ?? authFileJson.sfdxAuthUrl;
};
//# sourceMappingURL=sfdx-url.js.map