/*
 * Copyright (c) 2023, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { SfError, Messages } from '@salesforce/core';
import { getObject, getString } from '@salesforce/ts-types';
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-data', 'importApi');
/** makes the API request */
export const sendSObjectTreeRequest = (conn) => (sobject) => (rawContents) => 
// post request with to-be-insert sobject tree content
conn.request({
    method: 'POST',
    url: `/composite/tree/${sobject}`,
    body: rawContents,
    headers: {
        'content-type': 'application/json',
    },
});
export const transformRecordTypeEntries = async (conn, records) => {
    await Promise.all(records.map(async (record) => {
        const recordName = getString(record, 'RecordType.Name');
        if (recordName) {
            const targetRecordTypeId = (await conn.singleRecordQuery(`SELECT Id FROM RecordType WHERE Name = '${recordName}' AND SobjectType='${record.attributes.type}'`)).Id;
            delete record['RecordType'];
            record['RecordTypeId'] = targetRecordTypeId;
        }
        else if (getObject(record, 'RecordType') && !recordName) {
            throw messages.createError('error.noRecordTypeName');
        }
    }));
    return records;
};
/** handle an error throw by sendSObjectTreeRequest.  Always throws */
export const treeSaveErrorHandler = (error) => {
    if (error instanceof Error && 'errorCode' in error && error.errorCode === 'INVALID_FIELD') {
        const field = error.message.split("'")[1];
        const object = error.message.slice(error.message.lastIndexOf(' ') + 1, error.message.length);
        throw messages.createError('FlsError', [field, object]);
    }
    if (error instanceof Error) {
        throw SfError.wrap(error);
    }
    throw error;
};
export const parseDataFileContents = (filePath) => (contents) => {
    if (!contents) {
        throw messages.createError('dataFileEmpty', [filePath]);
    }
    return JSON.parse(contents).records;
};
/** look inside the response. If the hasErrors property is true, throw a formatted error.  Otherwise, extract the results property */
export const getResultsIfNoError = (filePath) => (response) => {
    if (response.hasErrors === true) {
        throw messages.createError('dataImportFailed', [filePath, JSON.stringify(response.results, null, 4)]);
    }
    return response.results;
};
//# sourceMappingURL=importCommon.js.map