/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { Messages, SfError } from '@salesforce/core';
import { stringify } from 'csv-stringify/sync';
import { parse } from 'csv-parse';
// max rows per file in Bulk 1.0
const BATCH_RECORDS_LIMIT = 10_000;
/// max characters/bytes per file in Bulk 1.0
const BATCH_BYTES_LIMIT = 10_000_000;
const POLL_FREQUENCY_MS = 5000;
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-data', 'batcher');
export class Batcher {
    conn;
    ux;
    constructor(conn, ux) {
        this.conn = conn;
        this.ux = ux;
    }
    /**
     * get and display the job status; close the job if completed
     *
     * @param jobId {string}
     * @param doneCallback
     */
    async fetchAndDisplayJobStatus(jobId, doneCallback) {
        const job = this.conn.bulk.job(jobId);
        const jobInfo = await job.check();
        this.bulkStatus(jobInfo, undefined, undefined, true);
        if (doneCallback) {
            doneCallback({ job: jobInfo });
        }
        return jobInfo;
    }
    /**
     *
     * Handles ux output and massaging the data by filtering out the $ from the response
     */
    bulkStatus(summary, results, batchNum, isJob) {
        this.ux.log('');
        if (batchNum) {
            this.ux.styledHeader(messages.getMessage('BulkBatch', [batchNum]));
        }
        const errorMessages = (results ?? []).flatMap((result) => result.errors);
        if (errorMessages.length > 0) {
            this.ux.styledHeader(messages.getMessage('BulkError'));
            errorMessages.forEach((errorMessage) => {
                this.ux.log(errorMessage);
            });
        }
        this.ux.styledHeader(isJob ? messages.getMessage('BulkJobStatus') : messages.getMessage('BatchStatus'));
        // remove url field (present if isJob)
        // Object.entries loses type info, but will match the original type
        const output = Object.fromEntries(Object.entries(summary).filter(([key]) => key !== '$'));
        this.ux.styledObject(output);
        return output;
    }
    /**
     * create and execute batches based on the record arrays; wait for completion response if -w flag is set with > 0 minutes
     * to get proper logging/printing to console pass the instance of UX that called this method
     *
     * @param job {Job}
     * @param records
     * @param sobjectType {string}
     * @param wait {number}
     */
    async createAndExecuteBatches(job, records, sobjectType, wait) {
        const batchesCompleted = 0;
        let batchesQueued = 0;
        const overallInfo = false;
        const batches = await splitIntoBatches(records);
        // The error handling for this gets quite tricky when there are multiple batches
        // Currently, we bail out early by calling an Error.exit
        // But, we might want to actually continue to the next batch.
        return (await Promise.all(batches.map(async (batch, i) => {
            const newBatch = job.createBatch();
            return new Promise((resolve, reject) => {
                // eslint-disable-next-line @typescript-eslint/no-floating-promises
                newBatch.on('error', (err) => {
                    // reword no external id error message to direct it to org user rather than api user
                    if (err.message.startsWith('External ID was blank')) {
                        err.message = messages.getMessage('ExternalIdRequired', [sobjectType]);
                        job.emit('error', err);
                    }
                    if (err.name === 'PollingTimeout') {
                        err.message = this.parseTimeOutError(err);
                        // using the reject method for all of the promises wasn't handling errors properly
                        // so emit a 'error' on the job.
                        job.emit('error', new SfError(err.message, 'Time Out', [], 69));
                    }
                    this.ux.spinner.stop('Error');
                });
                void newBatch.on('queue', 
                // eslint-disable-next-line @typescript-eslint/no-misused-promises
                async () => {
                    batchesQueued++;
                    if (batchesQueued === batches.length) {
                        /* jsforce clears out the id after close, but you should be able to close a job
                    after the queue, so add it back so future batch.check don't fail.*/
                        const id = job.id;
                        await job.close();
                        job.id = id;
                    }
                });
                if (!wait) {
                    // eslint-disable-next-line @typescript-eslint/no-floating-promises
                    newBatch.on('queue', 
                    // we're using an async method on an event listener which doesn't fit the .on method parameter types
                    // eslint-disable-next-line @typescript-eslint/no-misused-promises
                    async (batchInfo) => {
                        this.ux.log(messages.getMessage('CheckStatusCommand', [i + 1, batchInfo.jobId, batchInfo.id]));
                        const result = await newBatch.check();
                        if (result.state === 'Failed') {
                            reject(result.stateMessage);
                        }
                        else {
                            resolve(batchInfo);
                        }
                    });
                }
                else {
                    resolve(this.waitForCompletion(newBatch, batchesCompleted, overallInfo, i + 1, batches.length, wait));
                }
                void newBatch.execute(batch);
            });
        })));
    }
    /**
     * The timeout error handling is messy so to increase readability
     * break it out into it's own method
     *
     * @param err The timeout Error
     * @private
     */
    parseTimeOutError(err) {
        const jobIdIndex = err.message.indexOf('750');
        const batchIdIndex = err.message.indexOf('751');
        const message = messages.getMessage('TimeOut', [
            err.message.substr(jobIdIndex, 18),
            err.message.substr(batchIdIndex, 18),
        ]);
        this.ux.log('');
        process.exitCode = 69;
        return message;
    }
    /**
     * register completion event listeners on the batch
     * exposed for unit testing
     *
     * @param newBatch
     * @param batchesCompleted
     * @param overallInfo
     * @param batchNum
     * @param totalNumBatches
     * @param waitMins
     */
    async waitForCompletion(newBatch, batchesCompleted, overallInfo, batchNum, totalNumBatches, waitMins) {
        return new Promise((resolve, reject) => {
            void newBatch.on('queue', 
            // we're using an async method on an event listener which doesn't fit the .on method parameter types
            // eslint-disable-next-line @typescript-eslint/no-misused-promises
            async (batchInfo) => {
                const result = await newBatch.check();
                if (result.state === 'Failed') {
                    reject(result.stateMessage);
                }
                else if (!overallInfo) {
                    this.ux.log(messages.getMessage('PollingInfo', [POLL_FREQUENCY_MS / 1000, batchInfo.jobId]));
                    overallInfo = true;
                }
                this.ux.log(messages.getMessage('BatchQueued', [batchNum, batchInfo.id]));
                newBatch.poll(POLL_FREQUENCY_MS, waitMins * 60_000);
            });
            // we're using an async method on an event listener which doesn't fit the .on method parameter types
            // eslint-disable-next-line @typescript-eslint/no-misused-promises
            void newBatch.on('response', async (results) => {
                const summary = await newBatch.check();
                this.bulkStatus(summary, results, batchNum);
                batchesCompleted++;
                if (batchesCompleted === totalNumBatches) {
                    resolve(await this.fetchAndDisplayJobStatus(summary.jobId));
                }
            });
        });
    }
}
/**
 * registers the listener in charge of distributing all csv records into batches
 *
 * @param readStream - the read stream
 * @returns {Promise<Batches>}
 */
export const splitIntoBatches = async (readStream) => {
    // split all records into batches
    const batches = [];
    let batchIndex = 0;
    let batchBytes = 0;
    let batchHeaderBytes = 0;
    batches[batchIndex] = [];
    return new Promise((resolve, reject) => {
        const parser = parse({
            columns: true,
            // library option is snakecase
            // eslint-disable-next-line camelcase
            skip_empty_lines: true,
            bom: true,
        });
        readStream.pipe(parser);
        parser.on('data', (element) => {
            if (!batchHeaderBytes) {
                // capture header byte length
                batchHeaderBytes = Buffer.byteLength(stringify([Object.keys(element)]) + '\n', 'utf8');
                batchBytes = batchHeaderBytes;
            }
            // capture row byte length
            const rowBytes = Buffer.byteLength(stringify([Object.values(element)]) + '\n', 'utf8');
            if (batches[batchIndex].length === BATCH_RECORDS_LIMIT || rowBytes + batchBytes > BATCH_BYTES_LIMIT) {
                // TODO: we can start processing this batch here
                // we need event listeners to remove all of the `await new Promise`
                // next batch
                batchIndex++;
                batches[batchIndex] = [];
                // reset file size to just the headers
                batchBytes = batchHeaderBytes;
            }
            batchBytes += rowBytes;
            batches[batchIndex].push(element);
        });
        parser.on('error', (err) => {
            readStream.destroy();
            reject(SfError.wrap(err));
        });
        parser.on('end', () => {
            readStream.destroy();
            resolve(batches);
        });
    });
};
//# sourceMappingURL=batcher.js.map