/*
 * Copyright (c) 2022, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { Flags, loglevel, orgApiVersionFlagWithDeprecations, SfCommand } from '@salesforce/sf-plugins-core';
import { Messages } from '@salesforce/core/messages';
import { Package } from '@salesforce/packaging';
import { requiredHubFlag } from '../../../utils/hubFlag.js';
import { maybeGetProject } from '../../../utils/getProject.js';
// Import i18n messages
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-packaging', 'package_displayancestry');
export class PackageVersionDisplayAncestryCommand extends SfCommand {
    static summary = messages.getMessage('summary');
    static examples = messages.getMessages('examples');
    static deprecateAliases = true;
    static aliases = ['force:package:version:displayancestry'];
    static flags = {
        loglevel,
        'target-dev-hub': requiredHubFlag,
        'api-version': orgApiVersionFlagWithDeprecations,
        package: Flags.string({
            char: 'p',
            summary: messages.getMessage('flags.package.summary'),
            description: messages.getMessage('flags.package.description'),
            required: true,
        }),
        'dot-code': Flags.boolean({
            aliases: ['dotcode'],
            deprecateAliases: true,
            summary: messages.getMessage('flags.dot-code.summary'),
            description: messages.getMessage('flags.dot-code.description'),
        }),
        verbose: Flags.boolean({
            summary: messages.getMessage('flags.verbose.summary'),
        }),
    };
    async run() {
        const { flags } = await this.parse(PackageVersionDisplayAncestryCommand);
        const packageAncestry = await Package.getAncestry(flags.package, await maybeGetProject(), flags['target-dev-hub'].getConnection(flags['api-version']));
        const jsonProducer = packageAncestry.getJsonProducer();
        if (flags['dot-code']) {
            const dotProducer = packageAncestry.getDotProducer();
            const dotCodeResult = dotProducer.produce();
            if (flags.json) {
                return dotCodeResult;
            }
            else {
                this.log(dotCodeResult);
            }
        }
        else {
            if (packageAncestry.requestedPackageId?.startsWith('04t')) {
                const paths = packageAncestry.getLeafPathToRoot(packageAncestry.requestedPackageId);
                this.log(`${paths[0].map((p) => p.getVersion()).join(' -> ')} (root)`);
                this.log();
            }
            const treeProducer = packageAncestry.getTreeProducer(flags.verbose);
            if (!flags.json) {
                treeProducer.produce();
            }
        }
        return jsonProducer.produce();
    }
}
//# sourceMappingURL=displayancestry.js.map