/*
 * Copyright (c) 2022, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import ansis from 'ansis';
import { ux } from '@oclif/core';
import terminalLink from 'terminal-link';
import { UxBase } from './base.js';
import { Spinner } from './spinner.js';
import { table } from './table.js';
import styledObject from './styledObject.js';
/**
 * UX methods for plugins. Automatically suppress console output if outputEnabled is set to false.
 *
 * @example
 * ```
 * import { SfCommand, Ux } from '@salesforce/sf-plugins-core';
 * import { AnyJson } from '@salesforce/ts-types';
 *
 * class MyCommand extends SfCommand<AnyJson> {
 *   public async run(): Promise<AnyJson> {
 *     const ux = new Ux(!this.jsonEnabled());
 *   }
 * }
 *
 * ```
 */
export class Ux extends UxBase {
    spinner;
    outputEnabled;
    constructor({ jsonEnabled } = { jsonEnabled: false }) {
        super(!jsonEnabled);
        this.outputEnabled = !jsonEnabled;
        this.spinner = new Spinner(this.outputEnabled);
    }
    /**
     * Log a message to the console. This will be automatically suppressed if output is disabled.
     *
     * @param message Message to log. Formatting is supported.
     * @param args Args to be used for formatting.
     */
    log(message, ...args) {
        this.maybeNoop(() => ux.stdout(message, ...args));
    }
    /**
     * Log a message to stderr. This will be automatically suppressed if output is disabled.
     *
     * @param message Message to log. Formatting is supported.
     * @param args Args to be used for formatting.
     */
    logToStderr(message, ...args) {
        this.maybeNoop(() => ux.stderr(message, ...args));
    }
    /**
     * Log a warning message to the console. This will be automatically suppressed if output is disabled.
     *
     * @param message Warning message to log.
     */
    warn(message) {
        this.maybeNoop(() => ux.warn(message));
    }
    /**
     * Display a table to the console. This will be automatically suppressed if output is disabled.
     *
     * @param data Data to be displayed
     * @param columns Columns to display the data in
     * @param options Options for how the table should be displayed
     */
    table(data, columns, options) {
        this.maybeNoop(() => table(data, columns, { 'no-truncate': true, ...options }));
    }
    /**
     * Display a url to the console. This will be automatically suppressed if output is disabled.
     *
     * @param text text to display
     * @param uri URL link
     * @param params
     */
    url(text, uri, params = {}) {
        this.maybeNoop(() => ux.stdout(terminalLink(text, uri, { fallback: () => uri, ...params })));
    }
    /**
     * Display stylized JSON to the console. This will be automatically suppressed if output is disabled.
     *
     * @param obj JSON to display
     */
    styledJSON(obj, theme) {
        // Default theme if sf's theme.json does not have the json property set. This will allow us
        // to ship sf-plugins-core before the theme.json is updated.
        const defaultTheme = {
            key: 'blueBright',
            string: 'greenBright',
            number: 'blue',
            boolean: 'redBright',
            null: 'blackBright',
        };
        const mergedTheme = { ...defaultTheme, ...theme };
        this.maybeNoop(() => ux.stdout(ux.colorizeJson(obj, { theme: mergedTheme })));
    }
    /**
     * Display stylized object to the console. This will be automatically suppressed if output is disabled.
     *
     * @param obj Object to display
     * @param keys Keys of object to display
     */
    styledObject(obj, keys) {
        this.maybeNoop(() => ux.stdout(styledObject(obj, keys)));
    }
    /**
     * Display stylized header to the console. This will be automatically suppressed if output is disabled.
     *
     * @param text header to display
     */
    styledHeader(text) {
        this.maybeNoop(() => ux.stdout(ansis.dim('=== ') + ansis.bold(text) + '\n'));
    }
}
//# sourceMappingURL=ux.js.map